<?php

declare(strict_types=1);

namespace Drupal\joinup_distribution\EventSubscriber;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\joinup_distribution\Entity\DistributionInterface;
use Drupal\joinup_distribution\Exception\MissingDistributionParentException;
use Drupal\joinup_rdf\Entity\RdfSyncEntityInterface;
use Drupal\rdf_sync\Event\RdfSyncNormalizeEvent;
use Drupal\rdf_sync\RdfSyncMapper;
use EasyRdf\Resource;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Listens to normalization of entities for RDF format encoding.
 */
final class DistributionParentRdfSyncSubscriber implements EventSubscriberInterface {

  /**
   * Constructs a new event subscriber instance.
   *
   * @param \Drupal\rdf_sync\RdfSyncMapper $mapper
   *   The mapper service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file URL generator service.
   */
  public function __construct(
    protected RdfSyncMapper $mapper,
    protected FileUrlGeneratorInterface $fileUrlGenerator,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      RdfSyncNormalizeEvent::class => 'addDistributionParentRelation',
    ];
  }

  /**
   * Adds the dcat:distribution relationship from parent to distribution.
   *
   * @param \Drupal\rdf_sync\Event\RdfSyncNormalizeEvent $event
   *   The event object.
   */
  public function addDistributionParentRelation(RdfSyncNormalizeEvent $event): void {
    $entity = $event->getEntity();
    $normalizedArray = $event->getNormalizedArray();
    if ($entity instanceof DistributionInterface) {
      try {
        /** @var \Drupal\joinup_rdf\Entity\RdfSyncEntityInterface $parent */
        if ($parent = $entity->getParent()) {
          // @todo Remove these lines in ISAICP-8376. The URI should always be
          //   fetched with the getUri() method once RDF2Node is completed.
          if ($parent instanceof RdfSyncEntityInterface) {
            $parentUri = $parent->getUri();
          }
          else {
            // The parents have not been converted yet, we can get their ID
            // directly.
            $parentUri = $parent->id();
          }
          $normalizedArray[$parentUri]['http://www.w3.org/ns/dcat#distribution'][] = (new Resource($entity->getUri()))->toRdfPhp();
        }
      }
      catch (MissingDistributionParentException $exception) {
        // The parent is missing, so we cannot add the relationship. This can
        // happen during during complex operations like entities cleanup where
        // the order of the cleanup can be unpredictable.
      }

      // Export also all `access_url` values that are external URLs because RDF
      // sync normalizer handles the `target_id` generically assuming that all
      // entity reference fields are handled the same.
      // @todo Re-visit this in ISAICP-8821.
      // @see https://citnet.tech.ec.europa.eu/CITnet/jira/browse/ISAICP-8821.
      if (!$entity->get('access_url')->isEmpty()) {
        foreach ($entity->get('access_url')->referencedEntities() as $file) {
          $file_url = $file->getFileUri();
          if (UrlHelper::isExternal($file_url) && !UrlHelper::externalIsLocal($file_url, $GLOBALS['base_url'])) {
            $normalizedArray[$entity->getUri()]['http://www.w3.org/ns/dcat#accessURL'][] = (new Resource($file_url))->toRdfPhp();
          }
          else {
            $normalizedArray[$entity->getUri()]['http://www.w3.org/ns/dcat#accessURL'][] = (new Resource($this->fileUrlGenerator->generateAbsoluteString($file_url)))->toRdfPhp();
          }
        }
      }
    }
    $event->setNormalizedArray($normalizedArray);
  }

}
